"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, NotRequired, TypedDict


class AccountUpdaterOptionsTypedDict(TypedDict):
    response_code: Nullable[Literal["updated"]]
    r"""The type of response to simulate."""
    account_number: NotRequired[Nullable[str]]
    r"""When the `response_code` is set to `updated`, the payment method's account number will be updated to this value."""
    expiration_month: NotRequired[Nullable[str]]
    r"""When the `response_code` is set to `updated`, the payment method's expiration month will be updated to this value."""
    expiration_year: NotRequired[Nullable[str]]
    r"""When the `response_code` is set to `updated`, the payment method's expiration year will be updated to this value."""
    error_code: NotRequired[Nullable[str]]
    r"""The type of error code to simulate."""


class AccountUpdaterOptions(BaseModel):
    RESPONSE_CODE: Annotated[
        Annotated[
            OptionalNullable[Literal["updated"]],
            AfterValidator(validate_const("updated")),
        ],
        pydantic.Field(alias="response_code"),
    ] = "updated"
    r"""The type of response to simulate."""

    account_number: OptionalNullable[str] = UNSET
    r"""When the `response_code` is set to `updated`, the payment method's account number will be updated to this value."""

    expiration_month: OptionalNullable[str] = UNSET
    r"""When the `response_code` is set to `updated`, the payment method's expiration month will be updated to this value."""

    expiration_year: OptionalNullable[str] = UNSET
    r"""When the `response_code` is set to `updated`, the payment method's expiration year will be updated to this value."""

    error_code: OptionalNullable[str] = UNSET
    r"""The type of error code to simulate."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "response_code",
            "account_number",
            "expiration_month",
            "expiration_year",
            "error_code",
        ]
        nullable_fields = [
            "response_code",
            "account_number",
            "expiration_month",
            "expiration_year",
            "error_code",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from gr4vy.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from gr4vy.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, NotRequired, TypedDict


RouteType = Union[
    Literal[
        "round_trip",
        "one_way",
    ],
    UnrecognizedStr,
]


class AirlineLegTypedDict(TypedDict):
    arrival_airport: NotRequired[Nullable[str]]
    r"""Arrival airport code of leg. 3-letter ISO code according to IATA official directory."""
    arrival_at: NotRequired[Nullable[datetime]]
    r"""The date and time of travel in local time at the arrival airport."""
    arrival_city: NotRequired[Nullable[str]]
    r"""Arrival city name."""
    arrival_country: NotRequired[Nullable[str]]
    r"""Arrival country code in ISO 3166 format."""
    carrier_code: NotRequired[Nullable[str]]
    r"""3 character airline code as set by IATA."""
    carrier_name: NotRequired[Nullable[str]]
    r"""Name of the airline."""
    iata_designator: NotRequired[Nullable[str]]
    r"""Two-character IATA code of the airline."""
    icao_code: NotRequired[Nullable[str]]
    r"""Three-character ICAO code of the airline."""
    coupon_number: NotRequired[Nullable[str]]
    r"""Coupon number associated with the leg."""
    departure_airport: NotRequired[Nullable[str]]
    r"""Departure airport code of leg. 3-letter ISO code according to IATA official directory."""
    departure_at: NotRequired[Nullable[datetime]]
    r"""The date and time of travel in local time at the departure airport."""
    departure_city: NotRequired[Nullable[str]]
    r"""Departure city name."""
    departure_country: NotRequired[Nullable[str]]
    r"""Departure airport code of leg. 3-letter ISO code according to IATA official directory."""
    departure_tax_amount: NotRequired[Nullable[int]]
    r"""Departure tax amount charged by a country when a person is leaving the country."""
    fare_amount: NotRequired[Nullable[int]]
    r"""Amount of the ticket, for current leg of the trip, excluding taxes and fees."""
    fare_basis_code: NotRequired[Nullable[str]]
    r"""The alphanumeric code for the booking class of a ticket."""
    fee_amount: NotRequired[Nullable[int]]
    r"""Fee amount for current leg of the trip."""
    flight_class: NotRequired[Nullable[str]]
    r"""Indicates service class (first class, business class, etc.)."""
    flight_number: NotRequired[Nullable[str]]
    r"""Unique identifier of the flight number."""
    route_type: NotRequired[Nullable[RouteType]]
    r"""The route type of the flight."""
    seat_class: NotRequired[Nullable[str]]
    r"""Indicates seat class (first class, business class, etc.)."""
    stop_over: NotRequired[Nullable[bool]]
    r"""Indicates whether a stopover is allowed on this ticket."""
    tax_amount: NotRequired[Nullable[int]]
    r"""Amount of the taxes for current leg of the trip."""


class AirlineLeg(BaseModel):
    arrival_airport: OptionalNullable[str] = UNSET
    r"""Arrival airport code of leg. 3-letter ISO code according to IATA official directory."""

    arrival_at: OptionalNullable[datetime] = UNSET
    r"""The date and time of travel in local time at the arrival airport."""

    arrival_city: OptionalNullable[str] = UNSET
    r"""Arrival city name."""

    arrival_country: OptionalNullable[str] = UNSET
    r"""Arrival country code in ISO 3166 format."""

    carrier_code: OptionalNullable[str] = UNSET
    r"""3 character airline code as set by IATA."""

    carrier_name: OptionalNullable[str] = UNSET
    r"""Name of the airline."""

    iata_designator: OptionalNullable[str] = UNSET
    r"""Two-character IATA code of the airline."""

    icao_code: OptionalNullable[str] = UNSET
    r"""Three-character ICAO code of the airline."""

    coupon_number: OptionalNullable[str] = UNSET
    r"""Coupon number associated with the leg."""

    departure_airport: OptionalNullable[str] = UNSET
    r"""Departure airport code of leg. 3-letter ISO code according to IATA official directory."""

    departure_at: OptionalNullable[datetime] = UNSET
    r"""The date and time of travel in local time at the departure airport."""

    departure_city: OptionalNullable[str] = UNSET
    r"""Departure city name."""

    departure_country: OptionalNullable[str] = UNSET
    r"""Departure airport code of leg. 3-letter ISO code according to IATA official directory."""

    departure_tax_amount: OptionalNullable[int] = UNSET
    r"""Departure tax amount charged by a country when a person is leaving the country."""

    fare_amount: OptionalNullable[int] = UNSET
    r"""Amount of the ticket, for current leg of the trip, excluding taxes and fees."""

    fare_basis_code: OptionalNullable[str] = UNSET
    r"""The alphanumeric code for the booking class of a ticket."""

    fee_amount: OptionalNullable[int] = UNSET
    r"""Fee amount for current leg of the trip."""

    flight_class: OptionalNullable[str] = UNSET
    r"""Indicates service class (first class, business class, etc.)."""

    flight_number: OptionalNullable[str] = UNSET
    r"""Unique identifier of the flight number."""

    route_type: Annotated[
        OptionalNullable[RouteType], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The route type of the flight."""

    seat_class: OptionalNullable[str] = UNSET
    r"""Indicates seat class (first class, business class, etc.)."""

    stop_over: OptionalNullable[bool] = UNSET
    r"""Indicates whether a stopover is allowed on this ticket."""

    tax_amount: OptionalNullable[int] = UNSET
    r"""Amount of the taxes for current leg of the trip."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "arrival_airport",
            "arrival_at",
            "arrival_city",
            "arrival_country",
            "carrier_code",
            "carrier_name",
            "iata_designator",
            "icao_code",
            "coupon_number",
            "departure_airport",
            "departure_at",
            "departure_city",
            "departure_country",
            "departure_tax_amount",
            "fare_amount",
            "fare_basis_code",
            "fee_amount",
            "flight_class",
            "flight_number",
            "route_type",
            "seat_class",
            "stop_over",
            "tax_amount",
        ]
        nullable_fields = [
            "arrival_airport",
            "arrival_at",
            "arrival_city",
            "arrival_country",
            "carrier_code",
            "carrier_name",
            "iata_designator",
            "icao_code",
            "coupon_number",
            "departure_airport",
            "departure_at",
            "departure_city",
            "departure_country",
            "departure_tax_amount",
            "fare_amount",
            "fare_basis_code",
            "fee_amount",
            "flight_class",
            "flight_number",
            "route_type",
            "seat_class",
            "stop_over",
            "tax_amount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


InstrumentType = Union[
    Literal[
        "pan",
        "card_token",
        "redirect",
        "redirect_token",
        "googlepay",
        "applepay",
        "network_token",
        "plaid",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


ThreeDSecureStatus = Union[
    Literal[
        "setup_error",
        "error",
        "declined",
        "cancelled",
        "complete",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .method import Method
from .paymentservices import PaymentServices, PaymentServicesTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import (
    FieldMetadata,
    HeaderMetadata,
    QueryParamMetadata,
    validate_open_enum,
)
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Callable, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListPaymentServicesGlobalsTypedDict(TypedDict):
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListPaymentServicesGlobals(BaseModel):
    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""


class ListPaymentServicesRequestTypedDict(TypedDict):
    method: NotRequired[Nullable[Method]]
    r"""Return any payment service for this method."""
    cursor: NotRequired[Nullable[str]]
    r"""A pointer to the page of results to return."""
    limit: NotRequired[int]
    r"""The maximum number of items that are at returned."""
    deleted: NotRequired[Nullable[bool]]
    r"""Return any deleted payment service."""
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListPaymentServicesRequest(BaseModel):
    method: Annotated[
        Annotated[OptionalNullable[Method], PlainValidator(validate_open_enum(False))],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Return any payment service for this method."""

    cursor: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""A pointer to the page of results to return."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 20
    r"""The maximum number of items that are at returned."""

    deleted: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Return any deleted payment service."""

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "method",
            "cursor",
            "limit",
            "deleted",
            "merchant_account_id",
        ]
        nullable_fields = ["method", "cursor", "deleted"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ListPaymentServicesResponseTypedDict(TypedDict):
    result: PaymentServicesTypedDict


class ListPaymentServicesResponse(BaseModel):
    next: Callable[[], Optional[ListPaymentServicesResponse]]

    result: PaymentServices



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentmethodstatus import PaymentMethodStatus
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentServiceTokenTypedDict(TypedDict):
    id: str
    r"""The ID for the payment service token."""
    payment_method_id: str
    r"""The ID of the payment method used to generate this token"""
    payment_service_id: str
    r"""The ID of the payment method used to generate this token."""
    status: PaymentMethodStatus
    created_at: datetime
    r"""The date and time when this payment service token was first created in our system."""
    updated_at: datetime
    r"""The date and time when this payment service token was last updated in our system."""
    type: Literal["payment-service-token"]
    r"""Always `payment-service-token`."""
    approval_url: NotRequired[Nullable[str]]
    r"""The optional URL that the buyer needs to be redirected to to further authorize the token creation."""
    token: NotRequired[Nullable[str]]
    r"""The token value. Will be present if succeeded."""


class PaymentServiceToken(BaseModel):
    id: str
    r"""The ID for the payment service token."""

    payment_method_id: str
    r"""The ID of the payment method used to generate this token"""

    payment_service_id: str
    r"""The ID of the payment method used to generate this token."""

    status: Annotated[PaymentMethodStatus, PlainValidator(validate_open_enum(False))]

    created_at: datetime
    r"""The date and time when this payment service token was first created in our system."""

    updated_at: datetime
    r"""The date and time when this payment service token was last updated in our system."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["payment-service-token"]],
            AfterValidator(validate_const("payment-service-token")),
        ],
        pydantic.Field(alias="type"),
    ] = "payment-service-token"
    r"""Always `payment-service-token`."""

    approval_url: OptionalNullable[str] = UNSET
    r"""The optional URL that the buyer needs to be redirected to to further authorize the token creation."""

    token: OptionalNullable[str] = UNSET
    r"""The token value. Will be present if succeeded."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "approval_url", "token"]
        nullable_fields = ["approval_url", "token"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .googlepaypaymentoptioncontext import (
    GooglePayPaymentOptionContext,
    GooglePayPaymentOptionContextTypedDict,
)
from .mode import Mode
from .paymentoptioncontext import PaymentOptionContext, PaymentOptionContextTypedDict
from .walletpaymentoptioncontext import (
    WalletPaymentOptionContext,
    WalletPaymentOptionContextTypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ContextTypedDict = TypeAliasType(
    "ContextTypedDict",
    Union[
        WalletPaymentOptionContextTypedDict,
        GooglePayPaymentOptionContextTypedDict,
        PaymentOptionContextTypedDict,
    ],
)


Context = TypeAliasType(
    "Context",
    Union[
        WalletPaymentOptionContext, GooglePayPaymentOptionContext, PaymentOptionContext
    ],
)


class PaymentOptionTypedDict(TypedDict):
    method: str
    mode: Mode
    can_store_payment_method: bool
    can_delay_capture: bool
    type: Literal["payment-option"]
    icon_url: NotRequired[Nullable[str]]
    label: NotRequired[Nullable[str]]
    context: NotRequired[Nullable[ContextTypedDict]]


class PaymentOption(BaseModel):
    method: str

    mode: Annotated[Mode, PlainValidator(validate_open_enum(False))]

    can_store_payment_method: bool

    can_delay_capture: bool

    TYPE: Annotated[
        Annotated[
            Optional[Literal["payment-option"]],
            AfterValidator(validate_const("payment-option")),
        ],
        pydantic.Field(alias="type"),
    ] = "payment-option"

    icon_url: OptionalNullable[str] = UNSET

    label: OptionalNullable[str] = UNSET

    context: OptionalNullable[Context] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "icon_url", "label", "context"]
        nullable_fields = ["icon_url", "label", "context"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from gr4vy.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, NotRequired, TypedDict


RedirectPaymentMethodCreateMethod = Union[
    Literal[
        "abitab",
        "affirm",
        "afterpay",
        "alipay",
        "alipayhk",
        "arcuspaynetwork",
        "bacs",
        "bancontact",
        "banked",
        "bcp",
        "becs",
        "bitpay",
        "blik",
        "boleto",
        "boost",
        "cashapp",
        "clearpay",
        "dana",
        "dcb",
        "dlocal",
        "ebanx",
        "efecty",
        "eps",
        "everydaypay",
        "gcash",
        "gem",
        "gemds",
        "giropay",
        "givingblock",
        "gocardless",
        "gopay",
        "grabpay",
        "ideal",
        "kakaopay",
        "kcp",
        "khipu",
        "klarna",
        "latitude",
        "latitudeds",
        "laybuy",
        "linepay",
        "linkaja",
        "maybankqrpay",
        "mercadopago",
        "multibanco",
        "multipago",
        "netbanking",
        "nupay",
        "nequi",
        "oney_10x",
        "oney_12x",
        "oney_3x",
        "oney_4x",
        "oney_6x",
        "ovo",
        "oxxo",
        "p24",
        "pagoefectivo",
        "payid",
        "paymaya",
        "paypal",
        "paypalpaylater",
        "payto",
        "payvalida",
        "picpay",
        "pix",
        "pse",
        "rabbitlinepay",
        "rapipago",
        "razorpay",
        "redpagos",
        "scalapay",
        "sepa",
        "servipag",
        "shopeepay",
        "singteldash",
        "smartpay",
        "sofort",
        "spei",
        "stitch",
        "stripedd",
        "stripetoken",
        "tapi",
        "tapifintechs",
        "thaiqr",
        "touchngo",
        "truemoney",
        "trustly",
        "trustlyeurope",
        "upi",
        "venmo",
        "vipps",
        "waave",
        "webpay",
        "wechat",
        "wero",
        "yape",
        "zippay",
    ],
    UnrecognizedStr,
]
r"""The method to use, this can be any of the methods that support redirect requests."""


class RedirectPaymentMethodCreateTypedDict(TypedDict):
    r"""Create a transaction for an APM/LPM that requires a redirect."""

    method: RedirectPaymentMethodCreateMethod
    r"""The method to use, this can be any of the methods that support redirect requests."""
    country: str
    r"""The 2-letter ISO code of the country to use this payment method for. This is used to select the payment service to use."""
    currency: str
    r"""The ISO-4217 currency code to use this payment method for. This is used to select the payment service to use."""
    redirect_url: str
    r"""The redirect URL to redirect a buyer to after they have authorized the payment method."""
    buyer_id: NotRequired[Nullable[str]]
    r"""The `id` of a stored buyer to use Use this instead of the `buyer_external_identifier`."""
    buyer_external_identifier: NotRequired[Nullable[str]]
    r"""The `external_identifier` of a stored buyer to use. Use this instead of the `buyer_id`."""
    external_identifier: NotRequired[Nullable[str]]
    r"""The merchant identifier for this payment method."""


class RedirectPaymentMethodCreate(BaseModel):
    r"""Create a transaction for an APM/LPM that requires a redirect."""

    method: Annotated[
        RedirectPaymentMethodCreateMethod, PlainValidator(validate_open_enum(False))
    ]
    r"""The method to use, this can be any of the methods that support redirect requests."""

    country: str
    r"""The 2-letter ISO code of the country to use this payment method for. This is used to select the payment service to use."""

    currency: str
    r"""The ISO-4217 currency code to use this payment method for. This is used to select the payment service to use."""

    redirect_url: str
    r"""The redirect URL to redirect a buyer to after they have authorized the payment method."""

    buyer_id: OptionalNullable[str] = UNSET
    r"""The `id` of a stored buyer to use Use this instead of the `buyer_external_identifier`."""

    buyer_external_identifier: OptionalNullable[str] = UNSET
    r"""The `external_identifier` of a stored buyer to use. Use this instead of the `buyer_id`."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""The merchant identifier for this payment method."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "buyer_id",
            "buyer_external_identifier",
            "external_identifier",
        ]
        nullable_fields = [
            "buyer_id",
            "buyer_external_identifier",
            "external_identifier",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SettlementTypedDict(TypedDict):
    r"""A settlement record for a transaction."""

    id: str
    r"""The unique identifier for the record."""
    merchant_account_id: str
    r"""The merchant account this record belongs to."""
    created_at: datetime
    r"""The date and time the record was created, in ISO 8601 format."""
    updated_at: datetime
    r"""The date and time the record was last updated, in ISO 8601 format."""
    posted_at: datetime
    r"""The date and time the record was posted, in ISO 8601 format."""
    ingested_at: datetime
    r"""The date and time the record was ingested, in ISO 8601 format."""
    currency: str
    r"""ISO 4217 currency code."""
    amount: int
    r"""The total amount in the smallest currency unit (e.g. cents)."""
    commission: int
    r"""The commission amount deducted in the smallest currency unit."""
    payment_service_report_id: str
    r"""The report ID from the payment service."""
    payment_service_report_file_ids: List[str]
    r"""List of file IDs for the payment service report."""
    transaction_id: str
    r"""The transaction this record is associated with."""
    exchange_rate: NotRequired[Nullable[float]]
    r"""The exchange rate, if applicable."""
    interchange: NotRequired[Nullable[int]]
    r"""The interchange fee, if applicable, in the smallest currency unit."""
    markup: NotRequired[Nullable[int]]
    r"""The markup fee, if applicable, in the smallest currency unit."""
    scheme_fee: NotRequired[Nullable[int]]
    r"""The scheme fee, if applicable, in the smallest currency unit."""
    type: Literal["settlement"]
    r"""Always `settlement`."""


class Settlement(BaseModel):
    r"""A settlement record for a transaction."""

    id: str
    r"""The unique identifier for the record."""

    merchant_account_id: str
    r"""The merchant account this record belongs to."""

    created_at: datetime
    r"""The date and time the record was created, in ISO 8601 format."""

    updated_at: datetime
    r"""The date and time the record was last updated, in ISO 8601 format."""

    posted_at: datetime
    r"""The date and time the record was posted, in ISO 8601 format."""

    ingested_at: datetime
    r"""The date and time the record was ingested, in ISO 8601 format."""

    currency: str
    r"""ISO 4217 currency code."""

    amount: int
    r"""The total amount in the smallest currency unit (e.g. cents)."""

    commission: int
    r"""The commission amount deducted in the smallest currency unit."""

    payment_service_report_id: str
    r"""The report ID from the payment service."""

    payment_service_report_file_ids: List[str]
    r"""List of file IDs for the payment service report."""

    transaction_id: str
    r"""The transaction this record is associated with."""

    exchange_rate: OptionalNullable[float] = UNSET
    r"""The exchange rate, if applicable."""

    interchange: OptionalNullable[int] = UNSET
    r"""The interchange fee, if applicable, in the smallest currency unit."""

    markup: OptionalNullable[int] = UNSET
    r"""The markup fee, if applicable, in the smallest currency unit."""

    scheme_fee: OptionalNullable[int] = UNSET
    r"""The scheme fee, if applicable, in the smallest currency unit."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["settlement"]],
            AfterValidator(validate_const("settlement")),
        ],
        pydantic.Field(alias="type"),
    ] = "settlement"
    r"""Always `settlement`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "exchange_rate",
            "interchange",
            "markup",
            "scheme_fee",
            "type",
        ]
        nullable_fields = ["exchange_rate", "interchange", "markup", "scheme_fee"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .digitalwalletprovider import DigitalWalletProvider
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DigitalWalletCreateTypedDict(TypedDict):
    r"""Request body for registering a new digital wallet"""

    provider: DigitalWalletProvider
    merchant_name: str
    accept_terms_and_conditions: bool
    merchant_display_name: NotRequired[Nullable[str]]
    merchant_url: NotRequired[Nullable[str]]
    merchant_country_code: NotRequired[Nullable[str]]
    domain_names: NotRequired[List[str]]


class DigitalWalletCreate(BaseModel):
    r"""Request body for registering a new digital wallet"""

    provider: Annotated[
        DigitalWalletProvider, PlainValidator(validate_open_enum(False))
    ]

    merchant_name: str

    accept_terms_and_conditions: bool

    merchant_display_name: OptionalNullable[str] = UNSET

    merchant_url: OptionalNullable[str] = UNSET

    merchant_country_code: OptionalNullable[str] = UNSET

    domain_names: Optional[List[str]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "merchant_display_name",
            "merchant_url",
            "merchant_country_code",
            "domain_names",
        ]
        nullable_fields = [
            "merchant_display_name",
            "merchant_url",
            "merchant_country_code",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


DefinitionFieldFormat = Union[
    Literal[
        "text",
        "multiline",
        "number",
        "timezone",
        "boolean",
    ],
    UnrecognizedStr,
]
